@@ -392,7 +392,8 @@ class Discriminator(nn.Module):
         num_non_residual_layers = max(0, int(resolution) - 8)
         num_residual_layers = 8 - 3
 
-        features = list(map(lambda n: (n,  2 ** (fmap_inverse_coef - n)), range(resolution, 2, -1)))
+        non_residual_resolutions = range(min(8, resolution), 2, -1)
+        features = list(map(lambda n: (n,  2 ** (fmap_inverse_coef - n)), non_residual_resolutions))
         features = list(map(lambda n: (n[0], min(n[1], fmap_max)), features))
 
         if num_non_residual_layers == 0:
@@ -413,7 +414,7 @@ class Discriminator(nn.Module):
             ))
 
         self.residual_layers = nn.ModuleList([])
-        for (res, ((_, chan_in), (_, chan_out))) in zip(range(resolution, 2, -1), chan_in_out):
+        for (res, ((_, chan_in), (_, chan_out))) in zip(non_residual_resolutions, chan_in_out):
             image_width = 2 ** resolution
 
             hamburger = None
@@ -738,13 +739,14 @@ class Trainer():
 
     def train(self):
         assert exists(self.loader), 'You must first initialize the data source with `.set_data_src(<folder of images>)`'
+        device = torch.device(f'cuda:{self.rank}')
 
         if not exists(self.GAN):
             self.init_GAN()
 
         self.GAN.train()
-        total_disc_loss = torch.tensor(0.).cuda(self.rank)
-        total_gen_loss = torch.tensor(0.).cuda(self.rank)
+        total_disc_loss = torch.zeros([], device=device)
+        total_gen_loss = torch.zeros([], device=device)
 
         batch_size = math.ceil(self.batch_size / self.world_size)
 
@@ -767,7 +769,7 @@ class Trainer():
             latents = torch.randn(batch_size, latent_dim).cuda(self.rank)
 
             generated_images = G(latents)
-            fake_output, fake_aux_loss = D_aug(generated_images.clone().detach(), calc_aux_loss = True, detach = True, **aug_kwargs)
+            fake_output, fake_aux_loss = D_aug(generated_images.detach(), calc_aux_loss = True, detach = True, **aug_kwargs)
 
             image_batch = next(self.loader).cuda(self.rank)
             image_batch.requires_grad_()
@@ -780,16 +782,16 @@ class Trainer():
             disc_loss = divergence
 
             aux_loss = real_aux_loss + fake_aux_loss
-            self.last_recon_loss = aux_loss.clone().detach().item()
             disc_loss = disc_loss + aux_loss
 
             disc_loss = disc_loss / self.gradient_accumulate_every
             disc_loss.register_hook(raise_if_nan)
             disc_loss.backward()
 
-            total_disc_loss += divergence.detach().item() / self.gradient_accumulate_every
+            total_disc_loss += divergence
 
-        self.d_loss = float(total_disc_loss)
+        self.last_recon_loss = aux_loss.item()
+        self.d_loss = float(total_disc_loss.item() / self.gradient_accumulate_every)
 
         self.GAN.D_opt.step()
 
@@ -810,9 +812,9 @@ class Trainer():
             gen_loss.register_hook(raise_if_nan)
             gen_loss.backward()
 
-            total_gen_loss += loss.detach().item() / self.gradient_accumulate_every
+            total_gen_loss += loss 
 
-        self.g_loss = float(total_gen_loss)
+        self.g_loss = float(total_gen_loss.item() / self.gradient_accumulate_every)
 
         self.GAN.G_opt.step()
 
@@ -831,6 +833,9 @@ class Trainer():
             self.load(self.checkpoint_num)
             raise NanException
 
+        del total_disc_loss
+        del total_gen_loss
+
         # periodically save results
 
         if self.is_main:
